'
' COMKEY.BAS                    Sun  12-23-1990  21:12:58 
'
'
'                       (C) 1990, JERRY OWENS, DBA
'                       VANGUARD MANAGEMENT SYSTEMS
'                    
'                    P.O. Box 1582  Tupelo, MS 38801
'                     
'                              601-862-2346
'
'
'
'       Purpose: To provide expanded keyboard command options 
'                When called COMKEY waits for a key to be pressed
'                and then returns values from 1 to ...
'                depending upon the key pressed and which shift
'                keys were engaged at the time.   With the unshifted,
'                Shift, Control, and Alternate combinations, eight
'                possiblities are available for most keys.
'
'                In those cases where the keyboard does not respond
'                for a particular combination, COMKEY reserves a number
'                anyway to keep its number sequence in order.
'
'       Syntax:  cmd = COMKEY(ACode, SCode, Flag)
'
'                cmd will be the COMKEY code
'
'                ACode will be the ASCII code and SCode the Scancode
'                returned by the keyboard
'
'                Flag will indicated which shift and/or toggle keys
'                were engaged:
'
'                       Left Shift:      Flag AND 1
'                       Right Shift:     Flag AND 2
'                       Control Key:     Flag AND 4
'                       Alternate Key:   Flag AND 8
'                       Insert Key:      Flag AND 16
'                       CapsLock Key:    Flag AND 32
'                       NumLock Key:     Flag AND 64
'                       ScrolLock Key:   Flag AND 128
'
'               If a mouse button was pressed, COMKEY will be:
'                       Left Button:     1001
'                       MiddleButton:    1002
'                       Right Button:    1004
'
'
'       Usage:          cmd = COMKEY(ACode,SCode,Flag)
'                       SELECT CASE cmd
'                           CASE 1 to 8         ' they pressed an a
'                              .
'                              .
'                           CASE 399            ' CTRL ALT ESCAPE
'                              .
'                              .
'                           CASE 592            ' SHIFT CTRL ALT F12
'
'
DEFINT A-Z

DECLARE SUB GETKEY3(Mouse, AscCode, ScanCode, Lft, Mid, Rgh) 
DECLARE SUB GETKBD(Insert, CapsLock, NumLock, ScrolLock)     
DECLARE SUB GETKBD1(LSHIFT, RSHIFT, CTRL, ALT)               

'

FUNCTION COMKEY% (ACode%, SCode%, Flag%)
STATIC Table  AS STRING

        IF LEN(Table) <> 156 THEN GOSUB QWERTY
        
    SKey = 0: SKey2 = 0: Flag = 0: Mouse = 0
    CALL GETKEY3(Mouse, AscCode, ScanCode, Lft, Mid, Rgh)
    CALL GETKBD(Insert, CapsLock, NumLock, ScrolLock)
    CALL GETKBD1(LSHIFT, RSHIFT, CTRL, ALT)

    IF LSHIFT THEN SKey = SKey OR 1: Flag = Flag OR 1: Z$ = "SHF "
        IF RSHIFT THEN
            SKey = SKey OR 1
            Flag = Flag OR 2
            Z$ = "SHF "
        END IF

        IF CTRL THEN
            SKey = SKey OR 2
            Flag = Flag OR 4
            Z$ = Z$ + "CTL "
        END IF
       
        IF ALT THEN
            SKey = SKey OR 4
            Flag = Flag OR 8
            Z$ = Z$ + "ALT "
        END IF

        IF Insert THEN Flag = Flag OR 16
        IF CapsLock THEN Flag = Flag OR 32
        IF NumLock THEN Flag = Flag OR 64
        IF ScrolLock THEN Flag = Flag OR 128

        IF Lft THEN SKey2 = 1
        IF Lft THEN SKey2 = SKey2 OR 2
        IF Lft THEN SKey2 = SKey2 OR 4

        ACode = AscCode
        SCode = ScanCode

        '------------- done if mouse button pressed ----------------
         IF SKey2 THEN COMKEY = SKey2 + 1000: EXIT FUNCTION
        
        
        '------------------ otherwise ------------------------------
        T$ = CHR$(ScanCode)
        P = INSTR(1, Table, T$)
        SELECT CASE P
            CASE 1 TO 26        'the alaphabet
                COMKEY = (P - 1) * 8 + SKey + 1
                
            CASE 27 TO 46       'the numbers 1 thru 0 on top row
                IF P > 26 AND P < 37 THEN P = P - 26    'set reg codes to 1
                IF P > 36 AND P < 47 THEN P = P - 36    'set ALT codes to 1
                COMKEY = 209 + (P - 1) * 8 + SKey

                
            CASE 47 TO 63       ' Punctuation, Return/Enter & Space Bar
'---------------------------------------------------------------------------
' Ptr: 47  48  49  50  51  52  53  54  55  56  57  58  59   60   61  62  63
' Val: 41, 12, 13, 43, 26, 27, 39, 40, 51, 52, 53, 28, 57, 130, 131,224,166
'---------------------------------------------------------------------------
                IF P = 62 THEN
                    IF AscCode = 47 THEN        'keypad / shares scan code
                        COMKEY = 593 + SKey     'with Grey ENTER so...
                        GOTO Done               'set value to that following
                    ELSE                        'Function Keys & Exit
                        P = 12                  '...unless it WAS Grey ENTER
                    END IF
                END IF
                IF P = 63 THEN P = 12   'make ENTER same as RETURN
                IF P > 46 AND P < 60 THEN P = P - 46    'set index to 1
                IF P = 60 OR P = 61 THEN P = P - 58     'make it 2 or 3
                COMKEY = 289 + (P - 1) * 8 + SKey
                             
           
            CASE 64 TO 68       'ESCape, Tab, Backspace
'------------------------------------------------------------------------
' Ptr:   64  65  66   67   68
' Val:    1, 15, 14, 148, 165
'------------------------------------------------------------------------
               IF P > 63 AND P < 67 THEN P = P - 63
               IF P = 67 OR P = 68 THEN P = 2
               COMKEY = 393 + (P - 1) * 8 + SKey        'max val = 416
                           
           
            CASE 69 TO 98      'Cursor & editing
'------------------------------------------------------------------------
'  Ptr:     69  70  71  72  73  74  75  76  77  78
'  Val:     72, 80, 75, 77, 73, 81, 71, 79, 82, 83   ' unshifted and shift
'  Ptr:     79  80  81  82  83  84  85  86  87  88
'  Val:    141,145,115,116,132,118,119,117,146,147   ' ctrl key scancodes
'  Ptr:     89  90  91  92  93  94  95  96  97  98
'  Val:    152,160,155,157,153,161,151,159,162,163   ' alt key scancodes
'------------------------------------------------------------------------
               IF P > 68 AND P < 79 THEN P = P - 68  ' set pointer to 1
               IF P > 78 AND P < 89 THEN P = P - 78  ' for all three sets
               IF P > 88 AND P < 99 THEN P = P - 88  '
               COMKEY = 417 + (P - 1) * 8 + SKey        'max val = 496
                                                        '495 = CTRL ALT DEL
                                                        '496 = SHFT CTRL ALT DEL
                                                        'both will reboot system
                                                        'if not disabled
            
'            F1   F2   F3   F4   F5   F6   F7   F8   F9  F10  F11  F12
'---------------------------------------------------------------------
'                       unshifted
' Ptr:     99  100  101  102  103  104  105  106  107  108  109  110
' Val:     59,  60,  61,  62,  63,  64,  65,  66,  67,  68, 133, 134
'                       with SHIFT
' Ptr:    111  112  113  114  115  116  117  118  119  120  121  122
' Val:     84,  85,  86,  87,  88,  89,  90,  91,  92,  93, 135, 136
'                       with CTRL
' Ptr:    123  124  125  126  127  128  129  130  131  132  133  134
' Val:     94,  95,  96,  97,  98,  99, 100, 101, 102, 103, 137, 138
'                       with ALT
' Ptr:    135  136  137  138  139  140  141  142  143  144  145  146
' Val:    104, 105, 106, 107, 108, 109, 110, 111, 112, 113, 139, 140
'---------------------------------------------------------------------
            CASE 99 TO 110
                'IF P > 98 AND P < 109 THEN P = P - 98   ' set pointer to 1
                'IF P > 109 AND P < 122 THEN P = P - 109 ' set pointer to 1
                'IF P > 122 AND P < 134 THEN P = P - 122 ' set pointer to 1
                P = P - 98                               'set pointer to 1
                COMKEY = 497 + (P - 1) * 8 + SKey
                              
            CASE 111 TO 122
                P = P - 110
                COMKEY = 497 + (P - 1) * 8 + SKey
                
            CASE 123 TO 134     '
                P = P - 122
                COMKEY = 497 + (P - 1) * 8 + SKey
                
            CASE 135 TO 146
                P = P - 134
                COMKEY = 497 + (P - 1) * 8 + SKey        'max val = 592
                
           
'------------------------------------ Grey Key Pad Specials ---------------   
'          C/   A/      *    -         5     C*   C-   C+   C5
' Ptr:    147  148    149  150  151  152    153  154  155  156
' Val:    149, 164,    55,  74,  78,  76,   150, 142, 144, 143
'--------------------------------------------------------------------------
            CASE 147, 148     'shift & unshift covered with RETURN/ENTER above
                COMKEY = 593 + SKey
            CASE 149 TO 156
                IF P > 148 AND P < 153 THEN P = P - 148
                IF P > 152 THEN P = P - 152
                COMKEY = 601 + (P - 1) * 8 + SKey        'max val =
            CASE ELSE
                COMKEY = 0
        END SELECT
Done:
                
EXIT FUNCTION

QWERTY:
'         A  B  C  D  E, etc... these are the scancode values for A thru Z
Table$ = ""
Table$ = Table$ + CHR$(30) + CHR$(48) + CHR$(46) + CHR$(32) + CHR$(18) + CHR$(33)
Table$ = Table$ + CHR$(34) + CHR$(35) + CHR$(23) + CHR$(36) + CHR$(37)
Table$ = Table$ + CHR$(38) + CHR$(50) + CHR$(49) + CHR$(24) + CHR$(25)
Table$ = Table$ + CHR$(16) + CHR$(19) + CHR$(31) + CHR$(20) + CHR$(22)
Table$ = Table$ + CHR$(47) + CHR$(17) + CHR$(45) + CHR$(21) + CHR$(44)
  
'     and 0 thru 9
NUMBERS:                               '     ALTERNATE Scancodes
Table$ = Table$ + CHR$(2) + CHR$(3) + CHR$(4) + CHR$(5) + CHR$(6) + CHR$(7)
Table$ = Table$ + CHR$(8) + CHR$(9) + CHR$(10) + CHR$(11) + CHR$(120)
Table$ = Table$ + CHR$(121) + CHR$(122) + CHR$(123) + CHR$(124) + CHR$(125)
Table$ = Table$ + CHR$(126) + CHR$(127) + CHR$(128) + CHR$(129)

PUNCTUATION:
Table$ = Table$ + CHR$(41) + CHR$(12) + CHR$(13) + CHR$(43) + CHR$(26)
Table$ = Table$ + CHR$(27) + CHR$(39) + CHR$(40) + CHR$(51) + CHR$(52)
Table$ = Table$ + CHR$(53) + CHR$(28) + CHR$(57) + CHR$(130) + CHR$(131)
Table$ = Table$ + CHR$(224) + CHR$(166)

ESC.TAB.BKSPACE:
'
Table$ = Table$ + CHR$(1) + CHR$(15) + CHR$(14) + CHR$(148) + CHR$(165)
    
CURSOR.AND.EDITING.KEYS:

    ' unshifted and shift
Table$ = Table$ + CHR$(72) + CHR$(80) + CHR$(75) + CHR$(77) + CHR$(73) + CHR$(81) + CHR$(71) + CHR$(79) + CHR$(82) + CHR$(83)
        'ctrl
Table$ = Table$ + CHR$(141) + CHR$(145) + CHR$(115) + CHR$(116) + CHR$(132) + CHR$(118) + CHR$(119) + CHR$(117) + CHR$(146) + CHR$(147)
        'alt
Table$ = Table$ + CHR$(152) + CHR$(160) + CHR$(155) + CHR$(157) + CHR$(153) + CHR$(161) + CHR$(151) + CHR$(159) + CHR$(162) + CHR$(163)

    
FUNCTION.KEYS:
                        'unshifted   
Table$ = Table$ + CHR$(59) + CHR$(60) + CHR$(61) + CHR$(62) + CHR$(63) + CHR$(64) + CHR$(65) + CHR$(66) + CHR$(67) + CHR$(68) + CHR$(133) + CHR$(134)
                        'with SHIFT
Table$ = Table$ + CHR$(84) + CHR$(85) + CHR$(86) + CHR$(87) + CHR$(88) + CHR$(89) + CHR$(90) + CHR$(91) + CHR$(92) + CHR$(93) + CHR$(135) + CHR$(136)
                        'with CTRL
Table$ = Table$ + CHR$(94) + CHR$(95) + CHR$(96) + CHR$(97) + CHR$(98) + CHR$(99) + CHR$(100) + CHR$(101) + CHR$(102) + CHR$(103) + CHR$(137) + CHR$(138)
                        'with ALT
Table$ = Table$ + CHR$(104) + CHR$(105) + CHR$(106) + CHR$(107) + CHR$(108) + CHR$(109) + CHR$(110) + CHR$(111) + CHR$(112) + CHR$(113) + CHR$(139) + CHR$(140)


GREY.KEYPAD:
'           the Grey / shares scancode 224 with Grey Enter CTRL Grey /
'           is 149 and ALT Grey / is 164

Table$ = Table$ + CHR$(149) + CHR$(164) + CHR$(55) + CHR$(74) + CHR$(78) + CHR$(76) + CHR$(150) + CHR$(142) + CHR$(144) + CHR$(143)

RETURN
END FUNCTION

